function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
var __generator = this && this.__generator || function(thisArg, body) {
    var f, y, t, g, _ = {
        label: 0,
        sent: function() {
            if (t[0] & 1) throw t[1];
            return t[1];
        },
        trys: [],
        ops: []
    };
    return g = {
        next: verb(0),
        "throw": verb(1),
        "return": verb(2)
    }, typeof Symbol === "function" && (g[Symbol.iterator] = function() {
        return this;
    }), g;
    function verb(n) {
        return function(v) {
            return step([
                n,
                v
            ]);
        };
    }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while(_)try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [
                op[0] & 2,
                t.value
            ];
            switch(op[0]){
                case 0:
                case 1:
                    t = op;
                    break;
                case 4:
                    _.label++;
                    return {
                        value: op[1],
                        done: false
                    };
                case 5:
                    _.label++;
                    y = op[1];
                    op = [
                        0
                    ];
                    continue;
                case 7:
                    op = _.ops.pop();
                    _.trys.pop();
                    continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                        _ = 0;
                        continue;
                    }
                    if (op[0] === 3 && (!t || op[1] > t[0] && op[1] < t[3])) {
                        _.label = op[1];
                        break;
                    }
                    if (op[0] === 6 && _.label < t[1]) {
                        _.label = t[1];
                        t = op;
                        break;
                    }
                    if (t && _.label < t[2]) {
                        _.label = t[2];
                        _.ops.push(op);
                        break;
                    }
                    if (t[2]) _.ops.pop();
                    _.trys.pop();
                    continue;
            }
            op = body.call(thisArg, _);
        } catch (e) {
            op = [
                6,
                e
            ];
            y = 0;
        } finally{
            f = t = 0;
        }
        if (op[0] & 5) throw op[1];
        return {
            value: op[0] ? op[1] : void 0,
            done: true
        };
    }
};
import fs, { existsSync } from "fs";
import inquirer from "inquirer";
import { safeDump } from "js-yaml";
import path from "path";
import pc from "picocolors";
import rimraf from "rimraf";
import { CONF_FILE, TEMPLATE_FOLDER } from "../lib/constants.js";
import { printAndExit } from "../lib/utils/logger.js";
import { logInitHelp } from "./helps.js";
var ncp = import("ncp").ncp;
var askQuestions = function() {
    var questions = [
        {
            name: "NAME",
            type: "input",
            message: "What is the name of your first deck?",
            default: "deck"
        },
        {
            type: "list",
            name: "SIZE",
            message: "What is your card size?",
            choices: [
                "Poker",
                "Bridge",
                "Other"
            ]
        },
        {
            type: "input",
            name: "WIDTH",
            message: "What is your card width?",
            default: "825",
            when: function(res) {
                return res.SIZE === "Other";
            },
            filter: function(n) {
                return parseInt(n, 10);
            }
        },
        {
            type: "input",
            name: "HEIGHT",
            message: "What is your card height?",
            default: "1125",
            when: function(res) {
                return res.SIZE === "Other";
            },
            filter: function(n) {
                return parseInt(n, 10);
            }
        },
        {
            type: "checkbox",
            name: "FORMATS",
            message: "What formats do you want to generate??",
            default: "svg",
            choices: [
                {
                    name: "svg",
                    checked: true
                },
                "png",
                "pdf"
            ]
        }
    ];
    return inquirer.prompt(questions);
};
var touchSync = function(filename) {
    return fs.closeSync(fs.openSync(filename, "w"));
};
var getDefaultSizes = function(size) {
    switch(size){
        case "Bridge":
            return {
                width: 750,
                height: 1125
            };
        case "Poker":
        default:
            return {
                width: 825,
                height: 1125
            };
    }
};
var getConfig = function(param) {
    var NAME = param.NAME, SIZE = param.SIZE, WIDTH = param.WIDTH, HEIGHT = param.HEIGHT, FORMATS = param.FORMATS;
    var defaultSizes = getDefaultSizes(SIZE);
    var _obj;
    return _obj = {}, _obj["" + NAME] = {
        width: WIDTH ? WIDTH : defaultSizes.width,
        height: HEIGHT ? HEIGHT : defaultSizes.height,
        format: FORMATS
    }, _obj;
};
var dirEmpty = function(path) {
    return !existsSync(path) || existsSync(path) && !fs.readdirSync(path).length;
};
export var Init = /*#__PURE__*/ _asyncToGenerator(function(args) {
    var examplePath, destDir, answers, err, configPath, cleanupPaths;
    return __generator(this, function(_state) {
        switch(_state.label){
            case 0:
                if (args.help) {
                    logInitHelp();
                }
                examplePath = path.join(__dirname, "..", "..", "..", "examples");
                destDir = path.resolve(args.path);
                console.log(pc.green("Creating new deck in " + destDir));
                if (!dirEmpty(destDir)) {
                    printAndExit(pc.red("There's already a file at " + destDir + "."));
                }
                return [
                    4,
                    askQuestions()
                ];
            case 1:
                answers = _state.sent();
                _state.label = 2;
            case 2:
                _state.trys.push([
                    2,
                    4,
                    ,
                    5
                ]);
                return [
                    4,
                    ncp(examplePath, destDir)
                ];
            case 3:
                _state.sent();
                return [
                    3,
                    5
                ];
            case 4:
                err = _state.sent();
                printAndExit(err);
                return [
                    3,
                    5
                ];
            case 5:
                configPath = path.join(destDir, CONF_FILE);
                // Write config file
                fs.writeFileSync(configPath, safeDump(getConfig(answers)));
                // Rename example files
                fs.renameSync(path.join(destDir, "data", "example.csv"), path.join(destDir, "data", "" + answers.NAME + ".csv"));
                fs.renameSync(path.join(destDir, TEMPLATE_FOLDER, "example.js"), path.join(destDir, TEMPLATE_FOLDER, "" + answers.NAME + ".js"));
                cleanupPaths = [
                    "node_modules",
                    "package-lock.json",
                    "output"
                ];
                cleanupPaths.forEach(function(p) {
                    p = path.join(destDir, p);
                    if (existsSync(p)) {
                        rimraf.sync(p);
                    }
                });
                printAndExit("\n    Your project is now set up and ready to go. To start, run the following commands:\n\n    $ cd " + args.path + "\n    $ npm install\n    $ npm run build\n  ");
                return [
                    2
                ];
        }
    });
});
