import pc from "picocolors";
import _ from "lodash";
import parseArgs from "minimist";
import { Build } from "./cmd/build.js";
import { logDefaultHelp } from "./cmd/helps.js";
import { Init } from "./cmd/init.js";
process.on("SIGINT", function() {
    process.exit(0);
});
process.on("SIGTERM", function() {
    process.exit(0);
});
var defaultCmd = "build";
var commands = {
    init: Init,
    build: Build
};
var _obj;
var argv = parseArgs(process.argv.slice(2), (_obj = {
    alias: {
        h: "help",
        p: "path",
        P: "proof",
        w: "watch",
        v: "verbose",
        D: "decks",
        f: "formats"
    },
    string: [
        "decks",
        "formats"
    ],
    boolean: [
        "h",
        "P",
        "w",
        "v"
    ]
}, _obj["string"] = [
    "p"
], _obj.default = {
    path: "./",
    proof: false,
    watch: false,
    verbose: false
}, _obj));
var cmd = argv._[0];
var availableCmds = _.keys(commands);
if (argv.proof) {
    process.env.PROOF = true;
}
if (argv.verbose) {
    process.env.VERBOSE = true;
}
if (!cmd && argv.help) {
    logDefaultHelp(availableCmds);
}
if (!cmd) {
    var promise = commands[defaultCmd](argv);
    if (promise) {
        promise.then(function() {
            return process.exit(0);
        }).catch(function(e) {
            return console.error(pc.red(e.stack));
        });
    }
} else if (cmd && _.has(commands, cmd)) {
    var promise1 = commands[cmd](argv);
    if (promise1) {
        promise1.then(function() {
            return process.exit(0);
        }).catch(function(e) {
            console.error(pc.red(e.stack));
        });
    }
} else {
    console.warn(pc.yellow("Warning: Unknown Command: " + cmd));
    logDefaultHelp(availableCmds);
}
