function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _extends() {
    _extends = Object.assign || function(target) {
        for(var i = 1; i < arguments.length; i++){
            var source = arguments[i];
            for(var key in source){
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
    return _extends.apply(this, arguments);
}
var __generator = this && this.__generator || function(thisArg, body) {
    var f, y, t, g, _ = {
        label: 0,
        sent: function() {
            if (t[0] & 1) throw t[1];
            return t[1];
        },
        trys: [],
        ops: []
    };
    return g = {
        next: verb(0),
        "throw": verb(1),
        "return": verb(2)
    }, typeof Symbol === "function" && (g[Symbol.iterator] = function() {
        return this;
    }), g;
    function verb(n) {
        return function(v) {
            return step([
                n,
                v
            ]);
        };
    }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while(_)try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [
                op[0] & 2,
                t.value
            ];
            switch(op[0]){
                case 0:
                case 1:
                    t = op;
                    break;
                case 4:
                    _.label++;
                    return {
                        value: op[1],
                        done: false
                    };
                case 5:
                    _.label++;
                    y = op[1];
                    op = [
                        0
                    ];
                    continue;
                case 7:
                    op = _.ops.pop();
                    _.trys.pop();
                    continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                        _ = 0;
                        continue;
                    }
                    if (op[0] === 3 && (!t || op[1] > t[0] && op[1] < t[3])) {
                        _.label = op[1];
                        break;
                    }
                    if (op[0] === 6 && _.label < t[1]) {
                        _.label = t[1];
                        t = op;
                        break;
                    }
                    if (t && _.label < t[2]) {
                        _.label = t[2];
                        _.ops.push(op);
                        break;
                    }
                    if (t[2]) _.ops.pop();
                    _.trys.pop();
                    continue;
            }
            op = body.call(thisArg, _);
        } catch (e) {
            op = [
                6,
                e
            ];
            y = 0;
        } finally{
            f = t = 0;
        }
        if (op[0] & 5) throw op[1];
        return {
            value: op[0] ? op[1] : void 0,
            done: true
        };
    }
};
import yaml from "js-yaml";
import _ from "lodash";
import path from "path";
import { CONF_FILE } from "./constants.js";
import fsp from "./utils/fsp.js";
import { verboseLog } from "./utils/logger.js";
export var readConfig = /*#__PURE__*/ _asyncToGenerator(function(projectRoot, confOverride) {
    var configFile, conf, _, e;
    return __generator(this, function(_state) {
        switch(_state.label){
            case 0:
                configFile = path.join(projectRoot, CONF_FILE);
                verboseLog("...Looking for config in file " + configFile);
                _state.label = 1;
            case 1:
                _state.trys.push([
                    1,
                    3,
                    ,
                    4
                ]);
                _ = yaml.load;
                return [
                    4,
                    fsp.readFile(configFile, "utf8")
                ];
            case 2:
                conf = _.apply(yaml, [
                    _state.sent()
                ]);
                // Filter to keys if passed by CLI
                if (confOverride.decks) {
                    conf = filterConfigs(conf, confOverride.decks);
                }
                // Normalize conf
                conf = normalizeConfigs(conf);
                // Override formats
                if (confOverride.formats) {
                    conf = overrideFormats(conf, confOverride.formats);
                }
                return [
                    2,
                    conf
                ];
            case 3:
                e = _state.sent();
                console.error("...an error while reading the config: ", e);
                throw e;
            case 4:
                return [
                    2
                ];
        }
    });
});
var normalizeConfigs = function(conf) {
    return _.reduce(conf, function(result, value, key) {
        result[key] = processDeckConfig(value, key);
        return result;
    }, {});
};
var filterConfigs = function(conf, decks) {
    return _.pickBy(conf, function(_c, k) {
        return _.includes(decks, k);
    });
};
var overrideFormats = function(conf, formats) {
    var format = normalizeFormat(formats);
    return _.reduce(conf, function(result, value, key) {
        result[key] = _extends({}, value, {
            format: format
        });
        return result;
    }, {});
};
var processDeckConfig = function(deckConfig, deckKey) {
    if (!deckConfig.width) {
        throw "Parameter 'width' is required in deck " + deckKey;
    }
    if (!deckConfig.height) {
        throw "Parameter 'height' is required in deck " + deckKey;
    }
    var defaults = {
        data: "" + deckKey + ".csv",
        template: "" + deckKey + ".js",
        format: [
            "png"
        ],
        filename: "${deck}_${cardIndex}"
    };
    // Fix potentially bad data
    var conf = Object.assign(defaults, deckConfig);
    conf.format = normalizeFormat(conf.format);
    return conf;
};
var normalizeFormat = function(format) {
    if (Array.isArray(format)) {
        return format;
    }
    return [
        format
    ];
};
